
% find and learn, the only supervision is the bounding box for one image

% CONSTANTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
scale = .7;  % scale of Gabors, length = 17 pixels
norient = 15;  % number of orientations
epsilon = .1;  % allowed correlation between selected Gabors
T = 0.;        % threshold for responses
Totalsketch = 50;  % allowed number of selected Gabors   
Upperbound = 16.; % saturation value
sub = 2;    % subsampling rate for Gabor
Lrange = 3; % allowed shifting in location (times sub pixels)
Orange = 1; % allowed shifting in orientation

nstep = 3;  % number of iterations
LIK = 1.;  % use log-likelihood score. If LIK=0, then use active correlation
%LIK = 0.;  


% TRAINING IMAGES, only the first one needs to be labeled with bounding box
Iname = {'dear1.jpg' 'dear2.jpg' 'dear3.jpg' 'dear4.jpg' 'dear5.jpg'};
sx0 = 168; sy0 = 185;  % size of bounding box
xhere = 22;  yhere = 45; ihere = 0;  % position of the bounding box in image ihere

nimage = size(Iname, 2);
Sx = zeros(1, nimage); Sy = zeros(1, nimage); 
for (i = 1 : nimage)
    I{i} = randn(sx0, sy0); 
    J{i} = imresize(double(rgb2gray(imread(Iname{i}))), 1.); 
    [sx1, sy1] = size(J{i}); 
    Sx(i) = sx1; Sy(i) = sy1; % sizes of images
    tss{i} = zeros(sx1, sy1);  % log-likelihood score map for testing images
    Asym{i} = zeros(sx1, sy1);  % symbolic sketch on testing image
end
sym = zeros(size(I{1}));  % symbolic plot of selected Gabor

% FILTERING IMAGES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[allfilter, allsymbol] = makefilter(scale, norient);  % generate Gabor filters 
h = (size(allfilter{1}, 1)-1)/2;  % half size of Gabor
C = corr(allfilter, epsilon);    % generate the inhibition maps
allfilteredI = applyfilter(I, allfilter, T);  % filter training images
allfilteredJ = applyfilter(J, allfilter, T); 
allmaxed = applyfilter(J, allfilter, T);


% VARIABLES FOR MEX-C %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Mi = zeros(1, Totalsketch);  % orientation of selected Gabor
Mx = zeros(1, Totalsketch);  % position of selected Gabor
My = zeros(1, Totalsketch); 
Mm = zeros(1, Totalsketch);  % average response of selected Gabor

% MEX C %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
mex Cfindlearn.c;   % compile C code
Cfindlearn(nimage, norient, allfilteredI, allfilteredJ, allmaxed, C, h, sx0, sy0, T, Mi, Mx, My, Mm, allsymbol(1, :), Lrange, Orange, sub, Totalsketch, Upperbound, tss, sym, Asym, Sx, Sy, nstep, xhere, yhere, ihere, LIK);

% RESULTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure; 
subplot(2, 3, 1); imshow(-sym, []); 
for(i=1:nimage)
       subplot(2, 3, i+1, 'v6');  imshow(J{i}+Asym{i}*100, []);  % overlaid sketch 
end




